from __future__ import annotations

from .Any2AnyMultiChoice import *
from .Any2AnyRetrieval import *
from .Any2TextMultipleChoice import *
from .Clustering import *
from .ImageClassification import *
from .ImageMultilabelClassification import *
from .ImageTextPairClassification import *
from .VisualSTS import *
from .ZeroshotClassification import *
